from typing import Union, List
from enum import StrEnum, auto
from pydantic import BaseModel, PositiveInt, NonNegativeInt, conint, constr


'''
Enums
'''

class GameMode(StrEnum):
    EMPTY_STR = ""
    STANDARD = auto()
    SOLO = auto()
    ROYALE = auto()
    SQUAD = auto()
    CONSTRICTOR = auto()
    WRAPPED = auto()

class Source(StrEnum):
    TOURNAMENT = auto()
    LEAGUE = auto()
    ARENA = auto()
    CHALLENGE = auto()
    CUSTOM = auto()

class GameMap(StrEnum):
    EMPTY_STR = ""
    STANDARD = auto()
    EMPTY = auto()
    ARCADE_MAZE = auto()
    ROYALE = auto()
    SOLO_MAZE = auto()
    HZ_INNER_WALL = auto()
    HZ_RINGS = auto()
    HZ_COLUMNS = auto()
    HZ_RIVERS_BRIDGES = auto()
    HZ_SPIRAL = auto()
    HZ_SCATTER = auto()
    HZ_GROW_BOX = auto()
    HZ_EXPAND_BOX = auto()
    HZ_EXPAND_SCATTER = auto()

class GameMove(StrEnum):
    UP = auto()
    DOWN = auto()
    LEFT = auto()
    RIGHT = auto()


'''
Models
'''

class RoyaleRulesetSettings(BaseModel):
    shrinkEveryNTurns: NonNegativeInt

class SquadRulesetSettings(BaseModel):
    allowBodyCollisions: bool
    sharedElimination: bool
    sharedHealth: bool
    sharedLength: bool

class RulesetSettings(BaseModel):
    foodSpawnChance: conint(ge=0, le=100, strict=True)
    minimumFood: NonNegativeInt
    hazardDamagePerTurn: NonNegativeInt
    royale: RoyaleRulesetSettings
    squad: SquadRulesetSettings

class Ruleset(BaseModel):
    name: GameMode
    version: str
    settings: RulesetSettings


class Coordinate(BaseModel):
    x: NonNegativeInt
    y: NonNegativeInt


class BattlesnakeCustomizations(BaseModel):
    color: str
    head: str
    tail: str


class Battlesnake(BaseModel):
    id: str
    name: str
    health: conint(ge=0, le=100, strict=True)
    body: List[Coordinate]
    latency: str  # previous latency in milliseconds, str(int)
    head: Coordinate
    length: PositiveInt
    shout: constr(max_length=256)
    squad: str
    customizations: BattlesnakeCustomizations



class Board(BaseModel):
    height: PositiveInt
    width: PositiveInt
    food: List[Coordinate]
    hazards: List[Coordinate]
    snakes: List[Battlesnake]


class Game(BaseModel):
    id: str
    ruleset: Ruleset
    map: GameMap
    timeout: NonNegativeInt  # milliseconds
    source: Source


'''
GET /
'''
class GetResponse(BaseModel):
    apiversion: str
    author: Union[str, None] = None
    color: Union[str, None] = None
    head: Union[str, None] = None
    tail: Union[str, None] = None
    version: Union[str, None] = None


'''
POST /start
'''
class StartRequest(BaseModel):
    game: Game
    turn: NonNegativeInt
    board: Board
    you: Battlesnake

class StartResponse(BaseModel):
    pass


'''
POST /move
'''
class MoveRequest(BaseModel):
    game: Game
    turn: NonNegativeInt
    board: Board
    you: Battlesnake

class MoveResponse(BaseModel):
    move: GameMove
    shout: constr(max_length=256) = ""


'''
POST /end
'''
class EndRequest(BaseModel):
    game: Game
    turn: NonNegativeInt
    board: Board
    you: Battlesnake

class EndResponse(BaseModel):
    pass